#!/usr/bin/env python
#
# biolmai documentation build configuration file, created by
# sphinx-quickstart on Fri Jun  9 13:47:02 2017.
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

# If extensions (or modules to document with autodoc) are in another
# directory, add these directories to sys.path here. If the directory is
# relative to the documentation root, use os.path.abspath to make it
# absolute, like shown here.
#

# -- General configuration ---------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.viewcode",
    "sphinx_copybutton",
    "sphinx_inline_tabs",
    "sphinx.ext.autosectionlabel",
    "sphinxext.opengraph",
    "sphinx_design",
    "myst_parser",
    "sphinx_new_tab_link",
    "sphinx_reredirects",
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = ".rst"

# The master toctree document.
master_doc = "index"

# General information about the project.
project = "BioLM AI"
copyright = "2023, BioLM.ai"
author = "Nikhil Haas"

# The version info for the project you're documenting, acts as replacement
# for |version| and |release|, also used in various other places throughout
# the built documents.
#
# The short X.Y version.
version = '0.2.15'
# The full version, including alpha/beta/rc tags.
release = version

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = "en"

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = "sphinx"

# Dark mode syntax highlighting (Furo-specific)
pygments_dark_style = "material"

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = False


# -- Options for HTML output -------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "furo"

# Theme options are theme-specific and customize the look and feel of a
# theme further.  For a list of options available for each theme, see the
# documentation.
#
html_theme_options = {
    "top_of_page_button": None,
    "sidebar_hide_name": True,
    "light_logo": "biolm_logo_light.svg",
    "dark_logo": "biolm_logo_dark.svg",
    
    # Custom CSS variables for branding
    "light_css_variables": {
        # Primary brand colors - replace with your website's brand colors
        "color-brand-primary": "#2563eb",  # Replace with your main brand color
        "color-brand-content": "#2563eb",  # Links and interactive elements
        
        # Sidebar customization
        "color-sidebar-brand-text": "#1f2937",
        "color-sidebar-background": "#f8fafc",
        "color-sidebar-background-border": "#e2e8f0",
        
        # Additional brand colors
        "color-admonition-title-background--note": "#dbeafe",
        "color-admonition-title--note": "#1e40af",
        
        # Header and navigation
        "color-header-background": "#ffffff",
        "color-header-text": "#1f2937",
        
        # Code blocks
        "color-code-background": "#f1f5f9",
        "color-code-foreground": "#334155",
        
        # Font families to match your website exactly
        "font-stack": '"Inter", ui-sans-serif, system-ui, sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji"',
        "font-stack--monospace": '"Roboto Mono", ui-monospace, SFMono-Regular, Menlo, Monaco, Consolas, "Liberation Mono", "Courier New", monospace',
    },
    
    "dark_css_variables": {
        # Dark mode brand colors
        "color-brand-primary": "#60a5fa",  # Lighter version for dark mode
        "color-brand-content": "#60a5fa",
        
        # Dark sidebar
        "color-sidebar-background": "#1e293b",
        "color-sidebar-background-border": "#334155",
        "color-sidebar-brand-text": "#f1f5f9",
        
        # Dark mode code blocks
        "color-code-background": "#1e293b",
        "color-code-foreground": "#e2e8f0",
        
        # Additional dark mode styling
        "color-highlight-on-target": "#374151",
        
        # Font families to match your website exactly (same as light mode)
        "font-stack": '"Inter", ui-sans-serif, system-ui, sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji"',
        "font-stack--monospace": '"Roboto Mono", ui-monospace, SFMono-Regular, Menlo, Monaco, Consolas, "Liberation Mono", "Courier New", monospace',
    },
}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]

# Custom CSS files
html_css_files = [
    'custom.css',
]

# Add Google Fonts (Inter and Roboto Mono to match website)
html_css_files.extend([
    'https://fonts.googleapis.com/css2?family=Inter:wght@100;200;300;400;500;600;700;800;900&display=swap',
    'https://fonts.googleapis.com/css2?family=Roboto+Mono:wght@300;400;500;600;700&display=swap',
])

# Additional branding options
html_title = "BioLM AI Documentation"
html_short_title = "BioLM AI"

# Custom favicon (add your favicon.ico to _static folder)
# html_favicon = "_static/favicon.ico"

# Custom footer
html_show_sphinx = False
html_show_copyright = True

# SEO and metadata
html_meta = {
    "description": "Official documentation for BioLM AI - Advanced biological language models and APIs",
    "keywords": "BioLM, AI, machine learning, biology, protein, DNA, API",
    "author": "BioLM.ai Team",
    "viewport": "width=device-width, initial-scale=1",
}

# Social media cards / Open Graph
html_theme_options.update({
    "announcement": None,  # Add site-wide announcements here if needed
    "footer_icons": [
        {
            "name": "GitHub",
            "url": "https://github.com/yourusername/yourrepo",  # Update with your GitHub
            "html": """
                <svg stroke="currentColor" fill="currentColor" stroke-width="0" viewBox="0 0 16 16">
                    <path fill-rule="evenodd" d="M8 0C3.58 0 0 3.58 0 8c0 3.54 2.29 6.53 5.47 7.59.4.07.55-.17.55-.38 0-.19-.01-.82-.01-1.49-2.01.37-2.53-.49-2.69-.94-.09-.23-.48-.94-.82-1.13-.28-.15-.68-.52-.01-.53.63-.01 1.08.58 1.23.82.72 1.21 1.87.87 2.33.66.07-.52.28-.87.51-1.07-1.78-.2-3.64-.89-3.64-3.95 0-.87.31-1.59.82-2.15-.08-.2-.36-1.02.08-2.12 0 0 .67-.21 2.2.82.64-.18 1.32-.27 2-.27.68 0 1.36.09 2 .27 1.53-1.04 2.2-.82 2.2-.82.44 1.1.16 1.92.08 2.12.51.56.82 1.27.82 2.15 0 3.07-1.87 3.75-3.65 3.95.29.25.54.73.54 1.48 0 1.07-.01 1.93-.01 2.2 0 .21.15.46.55.38A8.013 8.013 0 0 0 16 8c0-4.42-3.58-8-8-8z"></path>
                </svg>
            """,
            "class": "",
        },
    ],
})


# -- Options for HTMLHelp output ---------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = "biolmaidoc"


# -- Options for LaTeX output ------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',
    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',
    # Additional stuff for the LaTeX preamble.
    #
    # 'preamble': '',
    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass
# [howto, manual, or own class]).
latex_documents = [
    (master_doc, "biolmai.tex", "BioLM AI Documentation", "Nikhil Haas", "manual"),
]


# -- Options for manual page output ------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [(master_doc, "biolmai", "BioLM AI Documentation", [author], 1)]


# -- Options for Texinfo output ----------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (
        master_doc,
        "biolmai",
        "BioLM AI Documentation",
        author,
        "biolmai",
        "One line description of project.",
        "Miscellaneous",
    ),
]

# Redirects
redirects = {
    "model-docs/esmfold/ESMFold_Additional": "https://biolm.ai/models/esmfold/",
    "model-docs/esmfold/ESMFold_API": "https://biolm.ai/models/esmfold/",
    "model-docs/esmfold/index": "https://biolm.ai/models/esmfold/",
    "model-docs/esm1v/ESM-1v_Additional": "https://biolm.ai/models/esm1v-all/",
    "model-docs/esm1v/ESM-1v_API": "https://biolm.ai/models/esm1v-all/",
    "model-docs/esm1v/index": "https://biolm.ai/models/esm1v-all/",
    "model-docs/esm2/ESM2_Additional": "https://biolm.ai/models/esm2-650m/",
    "model-docs/esm2/ESM2_API": "https://biolm.ai/models/esm2-650m/",
    "model-docs/esm2/index": "https://biolm.ai/models/esm2-650m/",
    "model-docs/progen2/ProGen2_Additional": "https://biolm.ai/models/progen2-medium/",
    "model-docs/progen2/ProGen2_API": "https://biolm.ai/models/progen2-medium/",
    "model-docs/progen2/index": "https://biolm.ai/models/progen2-medium/",
    "model-docs/esmif/ESM_InverseFold_Additional": "https://biolm.ai/models/esm-if1/",
    "model-docs/esmif/ESM_InverseFold_API": "https://biolm.ai/models/esm-if1/",
    "model-docs/esmif/index": "https://biolm.ai/models/esm-if1/",
    "model-docs/dnabert/DNABERT_Additional": "https://biolm.ai/models/dnabert2/",
    "model-docs/dnabert/classifier_ft": "https://biolm.ai/models/dnabert2/",
    "model-docs/dnabert/index": "https://biolm.ai/models/dnabert2/",
    "model-docs/biolmtox/BioLMTox_Additional": "https://biolm.ai/models/biolmtox/",
    "model-docs/biolmtox/BioLMTox_API": "https://biolm.ai/models/biolmtox/",
    "model-docs/biolmtox/index": "https://biolm.ai/models/biolmtox/",
}
